-- globals
cubes = 0
drawCubes = 0
drawCubes2 = 0

function love.load()
  -- window setup
  love.window.setFullscreen(true)
  love.window.setTitle("Ooga")
  love.window.setDisplaySleepEnabled(false)
  wW, wH = love.graphics.getWidth(), love.graphics.getHeight()
  love.mouse.setGrabbed(true)
  -- disable anti-aliasing
  love.graphics.setDefaultFilter('nearest','nearest')
  -- hump
  hump = {}
  hump.camera = require('Libs/hump-master/camera')
  hump.timer = require('Libs/hump-master/timer')
  hump.signal = require('Libs/hump-master/signal')
  hump.state = require('Libs/hump-master/gamestate')
  hump.class = require('Libs/hump-master/class')
  -- assets
  graphics = require('Graphics/defs')
  audio = require('Audio/defs')
  -- libs
  gridFuncs = require('Libs/ooga/gridFuncs')
  clientFuncs = require('Libs/ooga/mp/clientFuncs')
  instances = require('Libs/ooga/instancesDef')
  -- grid setup
  cube = love.graphics.newImage('Graphics/debugCube.png')
  cube_width = cube:getWidth()
  cube_height = cube:getHeight()
  cube_depth = cube_height / 2
	grid = gridFuncs.new3Dgrid(31,31,21,4,3,0,1,5,6)
  -- local player AKA client setup
  player = {}
  player.entity = instances.playerEntity('Bogdan',16,16,6)
  player.data = { canMove = true;
                       canJump = true;
                       jumping = false; 
                       moveTimer = hump.timer.new(); 
                       jumpTimer = hump.timer.new() 
                     }
  isoX = 0
  isoY = 0
  settings = {
      relativeMovement = false,
      camSpeed = 24,
      debug = true
  }
  shiftKey = false
  -- camera setup
  cam = hump.camera.new()
  camCycle = 0
  zoom = 1
  firstFrame = true
  -- logic setup
  logicCycle = 0
  gravityCycle = 0
end

function love.update(dt)
  -- mouse update
  mouseX, mouseY = love.mouse.getPosition()
  clientFuncs.updatePosR()
  -- timer update
  if not player.data.canMove then player.data.moveTimer:update(dt) end
  if player.data.jumping then player.data.jumpTimer:update(dt) end
  -- player update
  clientFuncs.globalMovement()
  clientFuncs.jump()
  clientFuncs.mineAndBuild()
  if gridFuncs.gridCheck(player.entity.pos.x,player.entity.pos.y,player.entity.pos.z - 1,0,false) then
    player.data.canJump = true
    player.data.jumping = false
    player.data.jumpTimer:clear()
  end
  -- cycle update
  camCycle = camCycle + 1*dt*settings.camSpeed
  logicCycle = camCycle + 1*dt
  if logicCycle >= 1/30 then
    logicCycle = 0
    gravityCycle = gravityCycle + 1
    if gravityCycle == 5 then
      gravityCycle = 0
      clientFuncs.gravity()
    end
  end
  -- camera update
  if camCycle >= 0.1 then
    -- zoom
    clientFuncs.camZoom()
    cam:zoomTo(zoom)
    -- get cam pos
    local x,y = cam:position()
    -- get the diff between the target pos and current pos
    local xDiff,yDiff = math.abs(isoX - x), math.abs(isoY - y)
    -- slow down if jumping or falling
    local m = 1
    if player.data.jumping or 
       gridFuncs.gridCheck(player.entity.pos.x,
                           player.entity.pos.y,
                           player.entity.pos.z - 1,0,true) then m = 0.3 end
    if xDiff < 5 then xDiff = 0 else xDiff = xDiff / (20 / (10*player.entity.stats.speed*m)) end
    if yDiff < 5 then yDiff = 0 else yDiff = yDiff / (20 / (10*player.entity.stats.speed*m)) end
    camCycle = 0
    -- move cam x pos
    if isoX > x then
      cam:lookAt(x + xDiff,y)
    else
      cam:lookAt(x - xDiff,y)
    end
    -- update cam pos
    local x,y = cam:position()
    -- move cam y pos
    if isoY > y then
      cam:lookAt(x,y + yDiff)
    else
      cam:lookAt(x,y - yDiff)
    end
  end
end

function love.draw()
  drawCubes = 0
  cam:attach()
  gridFuncs.gridDraw()
  cam:detach()
  love.graphics.setColor(1,1,1,1)
  if settings.debug then
    local x,y = cam:cameraCoords(isoX,isoY)
    love.graphics.print(' total cubes : ' .. cubes,0,0)
    love.graphics.print(' rendered cubes : ' .. drawCubes,0,12)
    love.graphics.print(' player [',0,24)
    love.graphics.print('     entity [',0,36)
    love.graphics.print('         name : ' .. player.entity.name,0,48)
    love.graphics.print('         pos [ x ' .. player.entity.pos.x .. 
                                      ' y ' .. player.entity.pos.y ..
                                      ' z ' .. player.entity.pos.z ..
                                      ' r ' .. player.entity.pos.r ..
                                      ' isoX ' .. isoX ..
                                      ' isoY ' .. isoY .. 
                                      ' X  ' .. x ..
                                      ' Y ' .. y .. ']',0,60)
    love.graphics.print('         stats [ hp ' .. player.entity.stats.hp ..
                                        ' exp ' .. player.entity.stats.exp ..
                                        ' lvl ' .. player.entity.stats.lvl ..
                                        ' mhp ' .. player.entity.stats.mhp ..
                                        ' mexp ' .. player.entity.stats.mexp ..
                                        ' speed ' .. player.entity.stats.speed .. ']',0,72)
    love.graphics.print('               ]',0,84)
    love.graphics.print('            ]',0,96)
  end
  if firstFrame then cam:lookAt(isoX,isoY); firstFrame = false end
end